<?php


/**
 * @version 23.11.dev
 * @package    Datafeeds for Joomla
 * @author     Bram <bram@brambring.nl>
 * @copyright  2023 Bram Brambring
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

 \defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Pagination\Pagination;


class DatafeedsControllerDatafeeds extends BaseController
{
    public function __construct($config = [])
    {
        parent::__construct($config);
        $this->db= Factory::getContainer()->get('DatabaseDriver');
    }
    public function display($cachable = false, $urlparams = false)
    {
        $app = Factory::getApplication();
        $context    = 'com_datafeed.datafeeds.list.';
        $lists=[];
        $formData=$app->input->getArray(['filter'=>'','list'=>'']);

        $filter=$formData['filter']??[];

        foreach (['f0','search','state'] as $field) {
          if ( isset($filter[$field])) {
            $app->setUserState($context.$field,$filter[$field]);
          } else {
            $filter[$field]=$app->getUserState($context.$field,  '', 'string');

          }
        }
        $list=$formData['list'];
        if ( isset($list['limit'])) {
          $app->setUserState($context.'limit',$list['limit']);
        } else {
           $list['limit']=$app->getUserState($context.'limit',  $app->get('list_limit'), 'int');
        }

        if ( isset($list['fullordering'])) {
          $app->setUserState($context.'fullordering',$list['fullordering']);
        } else {
           $list['fullordering']=$app->getUserState($context.'fullordering',  'feed ASC', 'string');
        }

        $lists['filter']=$filter;
        $lists['list']=$list;
        extract($filter);
        extract($list);
        $limitstart      = $app->getUserStateFromRequest($context.'limitstart', 'limitstart', 0, 'int');
        $where = [];
        if ($f0) {
            $where[] = "field0 = " .$this->db->Quote($this->db->escape($f0, true), false);
        }
        if ($search) {
            $where[] = ' feed LIKE '.$this->db->Quote('%'.$this->db->escape($search, true).'%', false);

        }

            switch ($state) {
                case 'P':
                    $where[] = 'published = 1';
                    break;
                case 'U':
                    $where[] = 'published = 0';
                    break;
                case 'A':
                    $where[] = 'published > 1';
                    break;
                case 'S':
                    $where[] = '(published = 1) AND (`last_update` + interval `freq` second< now())';
                    break;
                default:
                    ;
            }


        $where  = count($where) ?join(' AND ', $where) : 1;
        $query = $this->db->createQuery();
        $query->select('COUNT(*)')->from('#__datafeeds')->where($where);
        $this->db->setQuery($query);
        $total = $this->db->loadResult();
        $pageNav = new Pagination($total, $limitstart, $limit);
        $query = $this->db->createQuery();
        $query->setLimit($pageNav->limit, $pageNav->limitstart);
        $query->select('feed_id as id,  field0,feed, last_update,published,feed_url,  UNIX_TIMESTAMP(`last_update` + interval `freq` second) - UNIX_TIMESTAMP()  as old')
           ->from('#__datafeeds')->where($where)->order($fullordering) ;
        $this->db->setQuery($query);
        $rows = $this->db->loadObjectList();
        require_once(JPATH_COMPONENT.'/views/datafeeds.php');
        DatafeedsViewListfeeds::display($rows, $pageNav, $lists);
    }
    public function unpublish()
    {
        $input = Factory::getApplication()->input;
        $cid    = $input->get('cid', []);
        if ($cid) {
            $cids = implode(',', $cid);
            $q="update #__datafeeds set published=0,last_update='2022-01-01 00:01:01' where feed_id in ( $cids )";
            $this->db->setQuery($q);
            if (!$this->db->execute()) {
                $app->enqueueMessage(Text::_('COM_DATAFEEDS_OPERATION_FAILED') . ': ' . $this->db->getErrorMsg(), 'error');
                $this->setRedirect('index.php?option=com_datafeeds&c=dataitems');
                return;
            }
            $q="delete from  #__dataitems  where feed_id in ( $cids )";
            $this->db->setQuery($q);
            if (!$this->db->execute()) {
                $app->enqueueMessage(Text::_('COM_DATAFEEDS_OPERATION_FAILED') . ': ' . $this->db->getErrorMsg(), 'error');
                $this->setRedirect('index.php?option=com_datafeeds&c=dataitems');
                return;
            }
            $link = 'index.php?option=com_datafeeds&c=datafeeds';
            $this->setRedirect($link, Text::_('FEEDS_UNPUBLISHED'));
        }
    }
    public function publish()
    {
        $input = Factory::getApplication()->input;
                $cid    = $input->get('cid', []);
        if ($cid) {
            $cids = implode(',', $cid);
            $q="update #__datafeeds set published=1 where feed_id in ( $cids )";
            $this->db->setQuery($q);
            if (!$this->db->execute()) {
                $app->enqueueMessage(Text::_('COM_DATAFEEDS_OPERATION_FAILED') . ': ' . $this->db->getErrorMsg(), 'error');
                $this->setRedirect('index.php?option=com_datafeeds&c=dataitems');
            }
            $link = 'index.php?option=com_datafeeds&c=datafeeds';
            $this->setRedirect($link, Text::_('FEEDS_PUBLISHED'));
        }
    }
    public function cron()
    {
        $link='components/com_datafeeds/cron/feedcron.php';
        $this->setRedirect($link, Text::_('IMPORT_READY'));
    }

    public function delete()
    {

      $input = Factory::getApplication()->input;

      $cid    = $input->get('cid', []);
        if ($cid) {
            $cids = implode(',', $cid);
            $q="delete from  #__datafeeds  where feed_id in ( $cids )";
            $this->db->setQuery($q);
            if (!$this->db->execute()) {
                $app->enqueueMessage(Text::_('COM_DATAFEEDS_OPERATION_FAILED') . ': ' . $this->db->getErrorMsg(), 'error');
                $this->setRedirect('index.php?option=com_datafeeds&c=dataitems');
                return;
            }
            $q="delete from  #__dataitems  where feed_id in ( $cids )";
            $this->db->setQuery($q);
            if (!$this->db->execute()) {
                $app->enqueueMessage(Text::_('COM_DATAFEEDS_OPERATION_FAILED') . ': ' . $this->db->getErrorMsg(), 'error');
                $this->setRedirect('index.php?option=com_datafeeds&c=dataitems');
                return;
            }
            $link = 'index.php?option=com_datafeeds&c=datafeeds';
            $this->setRedirect('index.php?option=com_datafeeds&c=datafeeds',Text::_('FEEDS_REMOVED'));
        }
    }
}
