<?php


/**
 * @version 23.11.dev
 * @package    Datafeeds for Joomla
 * @author     Bram <bram@brambring.nl>
 * @copyright  2023 Bram Brambring
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

 \defined('_JEXEC') or die('Restricted access');
 

use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;

/**
 * Script file of datafeeds component
 */
class com_DatafeedsInstallerScript
{
    /**
     * method to install the component
     */
    function install($parent)
    {

        $cron = JPATH_ADMINISTRATOR . '/components/com_datafeeds/cron/';
        if (!File::exists($cron . 'feeds.php')) {
            $res = File::copy($cron . 'feeds-example.php', $cron . 'feeds.php');
        }
        Factory::getApplication()->enqueueMessage('Com_datafeeds installed', 'success');
    }

    /**
     * method to uninstall the component
     */
    function uninstall($parent)
    {
        Factory::getApplication()->enqueueMessage('com_datafeeds uninstalled<br/>Backup of feed customisations in the tmp/cron folder', 'success');
        return true;
    }

    /**
     * method to update the component
     */
    function update($parent)
    {
        ob_start();
        $this->deleteUnexistingFiles();
        $db = Factory::getContainer()->get('DatabaseDriver');
   
        $query = "DROP TABLE IF EXISTS `abc_#__dataitems`";
        $db->setQuery($query);
        $db->execute();

        $query = "SHOW COLUMNS IN `#__dataitems`";
        $db->setQuery($query);
        $db->execute();
        $dataitems = $db->loadAssocList('Field');
        $query = "SHOW COLUMNS IN `#__datafeeds`";
        $db->setQuery($query);
        $db->execute();
        $datafeeds = $db->loadAssocList('Field');
        $change = [
            'Prijs' => 'price',
            'prijs' => 'price',
            'groep' => 'field0',
            'menu_0' => 'field0',
            'menu_1' => 'field1',
            'menu_2' => 'field2',
            'menu_3' => 'field3',
            'menu_4' => 'field4',
            'menu_5' => 'field5',
            'menu_6' => 'field6',
            'menu_7' => 'field7',
            'menu_8' => 'field8',
            'menu_9' => 'field9',
            'menu_10' => 'field10',
            'menu_11' => 'field11',
            'menu_12' => 'field12',
            'menu_13' => 'field13',
            'menu_14' => 'field14',
            'menu_15' => 'field15',
            'menu_16' => 'field16',
            'menu_17' => 'field17',
            'menu_18' => 'field18',
            'menu_19' => 'field19',
            'Select0' => 'field0',
            'Select1' => 'field1',
            'Select2' => 'field2',
            'Select3' => 'field3',
            'Select4' => 'field4',
            'Select5' => 'field5',
            'Select6' => 'field6',
            'Select7' => 'field7',
            'Select8' => 'field8',
            'Select9' => 'field9',
            'Select10' => 'field10',
            'Select11' => 'field11',
            'Select12' => 'field12',
            'Select13' => 'field13',
            'Select14' => 'field14',
            'Select15' => 'field15',
            'Select16' => 'field16',
            'Select17' => 'field17',
            'Select18' => 'field18',
            'Select19' => 'field19',
            'href' => 'url',
        ];
        foreach ($change as $old => $new) {
            if (isset($dataitems[$old]) && !isset($dataitems[$new])) {
                echo "Updating dataitems $old to $new<br/>\n";
                $cur = $dataitems[$old];
                $type = $cur['Type'];
                $null = ($cur['Null'] == 'NO') ? 'Not' : '';
                $default = $db->quote($cur['Default']);
                $query = "ALTER IGNORE TABLE `#__dataitems` CHANGE `{$old}` `{$new}` {$type} {$null} null DEFAULT {$default}";
                $db->setQuery($query);
                $db->execute();
                $dataitems[$new] = $cur;
                unset($dataitems[$old]);
            }

            if (isset($datafeeds[$old]) && !isset($datafeeds[$new])) {
                echo "Updating datafeeds $old to $new<br/>\n";
                $cur = $datafeeds[$old];
                $type = $cur['Type'];
                $null = $cur['Null'] == 'NO' ? 'Not' : '';
                $default = $db->quote($cur['Default']);
                $query = "ALTER IGNORE TABLE `#__datafeeds` CHANGE `{$old}` `{$new}` {$type} {$null} null DEFAULT {$default}";
                $db->setQuery($query);
                $db->execute();
                $datafeeds[$new] = $cur;
                unset($datafeeds[$old]);
            }
        
        }
    
        for ($a = 10; $a <= 19; $a++) {
            $field = 'field' . $a;
            if (!isset($dataitems[$field])) {
                echo "Adding $field to database table dataitems<br/>\n";
                $query = "ALTER IGNORE TABLE `#__dataitems` ADD `$field` VARCHAR( 55 ) NOT NULL , ADD INDEX ( `$field` )";
                $db->setQuery($query);
                $db->execute();
            }
            if (!isset($datafeeds[$field])) {
                echo "Adding $field to database table datafeeds<br/>\n";
                $query = "ALTER IGNORE TABLE `#__datafeeds` ADD `$field` VARCHAR( 50 ) NOT NULL";
                $db->setQuery($query);
                $db->execute();
            }
        }
        $fields = ['input_encoding', 'UID'];
        foreach ($fields as $field) {
            if (!isset($datafeeds[$field])) {
                echo "Adding $field to database table datafeeds<br/>\n";
                $query = "ALTER IGNORE TABLE `#__datafeeds` ADD `$field` VARCHAR( 50 ) NOT NULL DEFAULT '\'\''";
                $db->setQuery($query);
                $db->execute();
            } else {
                if ($datafeeds[$field]['Default'] == '' || $datafeeds[$field]['Default'] == NULL) {
                    echo "Setting default value $field <br/>\n";
                    $type = $datafeeds[$field]['Type'];
                    $null = $datafeeds[$field]['Null'];
                    $default = $db->quote("''");
                    $null = ($null == 'NO') ? 'Not' : '';
                    $query = "ALTER IGNORE TABLE `#__datafeeds` CHANGE `$field` `$field` $type $null null DEFAULT $default";
                    $db->setQuery($query);
                    $db->execute();
                }
            }
        }
        if (isset($datafeeds['detect_encoding'])) {
            echo "Remove field detect_encoding<br/>\n";
            $query = "ALTER TABLE `#__datafeeds` DROP `detect_encoding`";
            $db->setQuery($query);
            $db->execute();
        }
        if (isset($datafeeds['data'])) {
            echo "Remove field data<br/>\n";
            $query = "ALTER TABLE `#__datafeeds` DROP `data`";
            $db->setQuery($query);
            $db->execute();
        }
        if ($datafeeds['feed_url']['Type'] != 'varchar(2048)') {
            echo "Set feed url field to varchar<br/>\n";
            $query = "ALTER TABLE `#__datafeeds` CHANGE `feed_url` `feed_url` VARCHAR( 2048 ) NOT NULL DEFAULT '0';";
            $db->setQuery($query);
            $db->execute();
        }
        if ($dataitems['price']['Type'] != 'decimal(10,2)') {
            echo "Set price field to decimal for sorting: {$dataitems['price']['Type']}<br/>\n";
            $query = "ALTER TABLE `#__dataitems` CHANGE `price` `price` DECIMAL( 10, 2 ) NOT NULL DEFAULT '0';";
            $db->setQuery($query);
            $db->execute();
        }
        $content=ob_get_flush();
        if ( $content) {
        Factory::getApplication()->enqueueMessage('Datafeeds database updated:'.$content, 'success');
        }
    }

    /**
     * method to run before an install/update/uninstall method
     */
    function preflight($type, $parent)
    {
        if ($type == "uninstall") {
            $app = Factory::getApplication();
            $tmp = $app->getCfg('tmp_path', $app->getCfg('config.tmp_path', JPATH_SITE . '/tmp'));
            if (Folder::exists($tmp)) {
                $cron = JPATH_ADMINISTRATOR . '/components/com_datafeeds/cron';
                $res = Folder::copy($cron, $tmp . '/cron', '', true);
            }
        }
    }

    /**
     * method to run after an install/update/uninstall method
     */
    function postflight($type, $parent)
    {
       
        if ($type == "install") {
            Factory::getApplication()->enqueueMessage('<h2>Quick start Quide</h2>
		<ol>
		<li>Click on the red link below to update the demo feeds<br/>
			<strong><a target="_blank" style="color:red" href="' . Uri::base() . 'index.php?option=com_datafeeds&c=cron">Click here to start/try importing feeds</a></strong>
		<li>Create a <em>datafeeds</em> menu item, just give it a title and leave the other tabs untouched
		<li>Create and position a datamenu module. the <em>default</em> layout is a good starting point.
		<li>You should have a affiliate shop with navigation on your site. Now you can add you own feeds.
		</ol>');
        }
       
        if ($type == "upgrade") {
            Factory::getApplication()->enqueueMessage('<h2>Please check your overrides</h2>
		<p> When upgrading from datafeeds version 4.0 to 4.2+</p>
		');
        }
    }

    /**
     * Delete files that should not exist
     *
     * @param bool  $dryRun          If set to true, will not actually delete files, but just report their status for use in CLI
     * @param bool  $suppressOutput   Set to true to suppress echoing any errors, and just return the $status array
     *
     * @return  array
     */
    public function deleteUnexistingFiles($dryRun = false, $suppressOutput = false)
    {
        $status = [
            'files_exist'     => [],
            'folders_exist'   => [],
            'files_deleted'   => [],
            'folders_deleted' => [],
            'files_errors'    => [],
            'folders_errors'  => [],
            'folders_checked' => [],
            'files_checked'   => [],
        ];

        $files = [
            '/administrator/components/com_datafeeds/cron/xml_fetch.inc',
            '/components/com_datafeeds/views/items/rss.php',
            '/components/com_datafeeds/views/items/view.xmlmap.php',
            '/components/com_datafeeds/views/items/view.feed.php',
            '/administrator/language/nl-NL/nl-NL.com_datafeeds.ini',
            '/administrator/language/nl-NL/nl-NL.com_datafeeds.sys.ini',
            '/administrator/components/com_datafeeds/controllers/daisycon.php',
            '/administrator/components/com_datafeeds/controllers/tradetracker.php',
            '/administrator/components/com_datafeeds/controllers/affili.php',
            '/administrator/components/com_datafeeds/controllers/m4n.php',
            '/administrator/components/com_datafeeds/views/daisycon.php',
            '/administrator/components/com_datafeeds/views/tradetracker.php',
            '/administrator/components/com_datafeeds/views/affili.php',
            '/administrator/components/com_datafeeds/views/m4n.php',
            '/administrator/components/com_datafeeds/cron/mysql.php',
            '/administrator/components/com_datafeeds/cron/req-test.php',
            '/administrator/components/com_datafeeds/cron/rss_cache.inc',
            '/administrator/components/com_datafeeds/cron/rss_fetch.inc',
            '/administrator/components/com_datafeeds/cron/rss_parse.inc',
            '/administrator/components/com_datafeeds/cron/rss_utils.inc',
            '/mod_datafeeds/Helper/DatamenuHelper.php',
            '/mod_datafeeds/Helper/DatafeedsmenuHelper.php'



        ];

        $folders = [];

        $status['files_checked'] = $files;
        $status['folders_checked'] = $folders;

        foreach ($files as $file) {
            if ($fileExists = File::exists(JPATH_ROOT . $file)) {
                $status['files_exist'][] = $file;

                if ($dryRun === false) {
                    if (File::delete(JPATH_ROOT . $file)) {
                        $status['files_deleted'][] = $file;
                    } else {
                        $status['files_errors'][] = Text::sprintf('FILES_JOOMLA_ERROR_FILE_FOLDER', $file);
                    }
                }
            }
        }


        foreach ($folders as $folder) {
            if ($folderExists = Folder::exists(JPATH_ROOT . $folder)) {
                $status['folders_exist'][] = $folder;

                if ($dryRun === false) {
                    if (Folder::delete(JPATH_ROOT . $folder)) {
                        $status['folders_deleted'][] = $folder;
                    } else {
                        $status['folders_errors'][] = Text::sprintf('FILES_JOOMLA_ERROR_FILE_FOLDER', $folder);
                    }
                }
            }
        }


        if ($suppressOutput === false && count($status['folders_errors'])) {
            echo implode('<br>', $status['folders_errors']);
        }

        if ($suppressOutput === false && count($status['files_errors'])) {
            echo implode('<br>', $status['files_errors']);
        }

        return $status;
    }
}
