<?php


/**
 * @version 23.11.dev
 * @package    Datafeeds for Joomla
 * @author     Bram <bram@brambring.nl>
 * @copyright  2023 Bram Brambring
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

 \defined('_JEXEC') or die('Restricted access');
 

use Joomla\CMS\MVC\View\GenericDataException;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Helper\ModuleHelper;


class DatafeedsController extends BaseController
{
    /**
     * Method to display the view.
     */
    public function display($cachable = false, $urlparams = false)
    {
        // Set a default view if none exists
        $input = Factory::getApplication()->input;
        if (!$input->get('view')) {
            $input->set('view', 'items');
        }
        $app = Factory::getApplication();
        $menus = $app->getMenu();
        $menu = $menus->getActive();
        if (!$input->get('layout', '', 'string') && isset($menu->query['layout'])) {
            $input->set('layout', $menu->query['layout']);
        }


        parent::display();
    }

    public function component()
    {
        $input = Factory::getApplication()->input;
        $input->set('view', 'items');
        $app = Factory::getApplication();
        $router = $app->getRouter();
        $router->setVar('format', '');
        $router->setVar('task', '');
        $document = Factory::getDocument();
        $document->setType('raw');
        parent::display();
    }

    public function module()
    {
    // too fetch module from an ajax request
    // not used by base component.
        $input = Factory::getApplication()->input;
        $module_id = $input->get('module_id', 0, 'int');
        $db = Factory::getContainer()->get('DatabaseDriver');
        $query = 'SELECT params'
            .' FROM #__modules AS m'
            .' WHERE m.id = '.(int) $module_id;
        $db->setQuery($query);

        if (null === ($params = $db->loadResult())) {
            throw new GenericDataException(Text::_('Error Loading Modules').$db->getErrorMsg(), 500);

            return false;
        }
        $a=json_decode($params);
        $a->ajax=1;
        $params=json_encode($a);
        $mod = ModuleHelper::getModule('mod_datafeeds');
        $mod->id = $module_id;
        $mod->params = $params;

        echo ModuleHelper::renderModule($mod);
    }

    public function hits()
    {
        $input = Factory::getApplication()->input;
        $item = $input->get('item', 0, 'int');
        $hits = $input->get('hits', 0, 'int');
        $Itemid = $input->get('c', 0, 'int');
        $db = Factory::getContainer()->get('DatabaseDriver');
        $update = 'UPDATE LOW_PRIORITY #__dataitems SET hits = ( hits + '.$hits.' ) WHERE items_id = '.$item;
        $query = 'select sql_no_cache * from #__dataitems WHERE items_id = '.$item;
        $db->setQuery($update);
        $db->execute(); //pech als die niet goedgaat
        $db->setQuery($query);
        $product = $db->loadAssoc();
        header('Content-type: text/xml');
        $xml = '<item id="'.$item.'">';
        if (is_array($product)) {
            foreach ($product as $k => $v) {
                $xml .= '<'.$k.'><![CDATA['.$v.']]></'.$k.'>'."\n";
            }
        }
        if ($Itemid) {
            $producturl = rtrim(Uri::Base(), '/').Route::_("index.php?option=com_datafeeds&view=items&Itemid=$Itemid&qt=".urlencode($product['title']));
            $xml .= '<producturl><![CDATA['.$producturl.']]></producturl>'."\n";
        }

        $xml .= '</item>';

        echo '<?xml version="1.0" encoding="UTF-8"?'.'>';
        echo "\n";
        echo $xml;
        exit;
    }

    public function go()
    {
        $app = Factory::getApplication();
        $input=$app->input;
        $db = Factory::getContainer()->get('DatabaseDriver');
        $item = $input->get('item', 0, 'int');
        if ($item and is_numeric($item)) {
            $query = 'select sql_no_cache url,feed_id as feed,title from #__dataitems WHERE items_id = '.$item;
            $update = 'UPDATE LOW_PRIORITY #__dataitems'.' SET hits = ( hits + 1 ) WHERE items_id = '.$item;

            $db->setQuery($query);
            $product = $db->loadObject();

            if ($product && $product->url) {
                $db->setQuery($update);
                $db->execute(); //pech als die niet goedgaat
                $p = $input->get('p', 'datafeed', 'string');
                $p .= '-'.$product->feed;
                $p .= '-'.$product->title;
                $p=urlencode($p);
                $product->url = str_replace(['&amp;', 'DF-REF'], ['&', $p], $product->url);
                $app->redirect($product->url, false);
            } else {
                /*this should never happen on 'normal' sites*/
                $app->redirect('/', false);
            }
        }
        $app->redirect('/', false);
    }

    public function search()
    {
        $a = ['q1', 'q2', 'q3', 'q4', 'q5', 'q6', 'q7', 'q8', 'q9', 'areas','min', 'max', 'mm', 'so', 'q', 'qt','sort', 'order'];
        $app = Factory::getApplication();
        $input=$app->input;
        $mode= Factory::getConfig()->get('sef', false);
        $post['option'] = 'com_datafeeds';
        $post['Itemid'] = $input->post->get('Itemid', null, 'int');
        foreach ($a as $v) {
            if ($q = $input->post->get($v, '', 'string')) {
                if ($mode) {
                    $post[$v] = trim($q);
                } else {
                    $post[$v] =trim(urldecode($q)); //not sure thi is still needed
                }
            }
        }

        $uri = Uri::getInstance();
        $uri->setQuery($post);

        $link = Route::_('index.php'.$uri->toString(['query', 'fragment']), false);
        $app->redirect($link, false);
    }
}
